package com.xkrs.service;

import com.xkrs.model.entity.FirePointEntity;
import com.xkrs.model.qo.FirePointQo;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author XinYi Song
 */
public interface FirePointService {

    /**
     * 添加火点信息
     *
     * @param firePointQo
     * @return
     */
    boolean insertFirePoint(FirePointQo firePointQo);

    /**
     * 查询今天的火点信息
     *
     * @param
     * @return
     */
    List<FirePointEntity> selectTodayFirePoint(String countyCode);

    /**
     * 查询近一周天的火点信息
     *
     * @return
     */
    List<FirePointEntity> selectFirePointBetweenSeven();

    /**
     * 查询近一个月的火点信息
     *
     * @return
     */
    List<FirePointEntity> selectFirePointByMonth(String countyCode);

    /**
     * 查询近一个月的火点数量
     *
     * @return
     */
    String selectFirePointNumMonth();

    /**
     * 查询近一天的火点数量
     *
     * @return
     */
    String selectFirePointNumDay();

    /**
     * 查询
     *
     * @return
     */
    String selectFirePointNumWeek();

    /**
     * download vip user's fire point
     *
     * @param request
     * @param response
     */
    String downloadVipUserFilePoint(HttpServletRequest request, HttpServletResponse response);

    String downloadFirePoint(HttpServletRequest request, HttpServletResponse response, String ids);

    /**
     * 动态多条件查询火点列表
     *
     * @param code          区划编码，可以是省市区县街道任意编码，如果传需要至少两位。不传代表查询全国火点
     * @param startTime     过滤火点开始卫星时间，不传代表不指定开始时间。后台会有限制。
     * @param endTime       过滤火点结束卫星时间，不传代表不指定结束时间。
     * @param satelliteType 过滤传感器类型，不传代表查询全部类型的传感器
     * @param landType      过滤地物类型，不传代表查询全部地物类型
     * @return 符合条件的火点列表
     */
    List<FirePointEntity> queryFirePoint(String code, String startTime, String endTime, String satelliteType, String landType);

}
