package com.xkrs.straw.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.straw.dao.FirePointChannelConfigDao;
import com.xkrs.straw.dao.FirePointOrdinaryDao;
import com.xkrs.straw.dao.FirePointPreciseDao;
import com.xkrs.straw.helper.GeoCodeHelper;
import com.xkrs.straw.model.entity.FirePointChannelConfigEntity;
import com.xkrs.straw.model.entity.FirePointOrdinaryEntity;
import com.xkrs.straw.model.entity.FirePointPreciseEntity;
import com.xkrs.straw.model.qo.AllFirePointQo;
import com.xkrs.straw.service.DispatchFirePointService;
import com.xkrs.straw.utils.DispatchFirePointUtils;
import com.xkrs.utils.DateTimeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@Service
public class DispatchFirePointServiceImpl implements DispatchFirePointService {

    public static Logger log = LoggerFactory.getLogger(DispatchFirePointServiceImpl.class);

    private final Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private FirePointChannelConfigDao channelConfigDao;

    @Resource
    private FirePointOrdinaryDao firePointOrdinaryDao;

    @Resource
    private FirePointPreciseDao firePointPreciseDao;

    @Override
    public String debug() {

//        FirePointChannelConfigBean firePointChannelConfigBean = new FirePointChannelConfigBean();
//        firePointChannelConfigBean.setChannelName(FirePointChannelConfigBean.Precise);
//        List<FirePointChannelConfigBean.ChannelConfig> configList = new ArrayList<>();
//
//        FirePointChannelConfigBean.ChannelConfig channelConfig1 = new FirePointChannelConfigBean.ChannelConfig();
//        channelConfig1.setStartTime("2022-01-01 00:00:00");
//        channelConfig1.setEndTime("2022-09-01 00:00:00");
//        configList.add(channelConfig1);
//
//        FirePointChannelConfigBean.ChannelConfig channelConfig2 = new FirePointChannelConfigBean.ChannelConfig();
//        channelConfig2.setStartTime("2022-10-01 00:00:00");
//        channelConfig2.setEndTime("2022-11-01 00:00:00");
//        configList.add(channelConfig2);
//
//        firePointChannelConfigBean.setConfigList(configList);
//        String serialize = JsonUtils.<FirePointChannelConfigBean>serialize(firePointChannelConfigBean);
//        FirePointChannelConfigEntity firePointChannelConfigEntity = new FirePointChannelConfigEntity();
//        firePointChannelConfigEntity.setJsonContent(serialize);
//        channelConfigDao.save(firePointChannelConfigEntity);
        return "null";
    }

    @Override
    public String insertFirePointChannelOrdinary(AllFirePointQo firePointQo) {

        String fireCode = firePointQo.getFireCode();
        Long countyCode = firePointQo.getCountyCode();
        String countyName = firePointQo.getCountyName();
        Long satelliteTimeTs = firePointQo.getSatelliteTimeTs();
        Double longitude = firePointQo.getLongitude();
        Double latitude = firePointQo.getLatitude();
        String satelliteType = firePointQo.getSatelliteType();
        String landType = firePointQo.getLandType();
        String confidence = firePointQo.getConfidence();
        String fireImage = firePointQo.getFireImage();
        String satelliteImage = firePointQo.getSatelliteImage();
        String townCode = firePointQo.getTownCode();

        LocalDateTime satelliteLocalDateTime = LocalDateTime.ofEpochSecond(satelliteTimeTs, 0, ZoneOffset.ofHours(8));
        String satelliteTime = DateTimeUtils.localDateTimeToString(satelliteLocalDateTime);//卫星时间
        String addTime = DateTimeUtils.localDateTimeToString(LocalDateTime.now());//添加时间

        //普通渠道来的火点直接入库普通火点库
        FirePointOrdinaryEntity firePointOrdinaryEntity = new FirePointOrdinaryEntity();
        firePointOrdinaryEntity.setFireCode(fireCode);
        firePointOrdinaryEntity.setCountyCode(String.valueOf(countyCode));
        firePointOrdinaryEntity.setCountyName(countyName);
        firePointOrdinaryEntity.setSatelliteTime(satelliteTime);
        firePointOrdinaryEntity.setLongitude(longitude);
        firePointOrdinaryEntity.setLatitude(latitude);
        firePointOrdinaryEntity.setSatelliteType(satelliteType);
        firePointOrdinaryEntity.setLandType(landType);
        firePointOrdinaryEntity.setConfidence(confidence);
        firePointOrdinaryEntity.setFireImage(fireImage);
        firePointOrdinaryEntity.setSatelliteImage(satelliteImage);
        firePointOrdinaryEntity.setTownCode(townCode);
        firePointOrdinaryEntity.setAddTime(addTime);
        bindFirePointAddress(firePointOrdinaryEntity);
        firePointOrdinaryDao.save(firePointOrdinaryEntity);

        List<FirePointChannelConfigEntity> firePointChannelConfigEntityList = channelConfigDao.findAll();
        boolean inPreciseTimeRange = DispatchFirePointUtils.checkIfInPreciseTimeRange(satelliteLocalDateTime, firePointChannelConfigEntityList);
        //如果现在不属于精准渠道的时间范围，那么普通渠道来的火点就入库精准火点库
        if (!inPreciseTimeRange) {
            FirePointPreciseEntity firePointPreciseEntity = new FirePointPreciseEntity();
            firePointPreciseEntity.setFireCode(fireCode);
            firePointPreciseEntity.setCountyCode(String.valueOf(countyCode));
            firePointPreciseEntity.setCountyName(countyName);
            firePointPreciseEntity.setSatelliteTime(satelliteTime);
            firePointPreciseEntity.setLongitude(longitude);
            firePointPreciseEntity.setLatitude(latitude);
            firePointPreciseEntity.setSatelliteType(satelliteType);
            firePointPreciseEntity.setLandType(landType);
            firePointPreciseEntity.setConfidence(confidence);
            firePointPreciseEntity.setFireImage(fireImage);
            firePointPreciseEntity.setSatelliteImage(satelliteImage);
            firePointPreciseEntity.setTownCode(townCode);
            firePointPreciseEntity.setAddTime(addTime);
            bindFirePointAddress(firePointPreciseEntity);
            firePointPreciseDao.save(firePointPreciseEntity);
        }

        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "添加成功", locale);
    }

    @Override
    public String insertFirePointChannelPrecise(AllFirePointQo firePointQo) {

        String fireCode = firePointQo.getFireCode();
        Long countyCode = firePointQo.getCountyCode();
        String countyName = firePointQo.getCountyName();
        Long satelliteTimeTs = firePointQo.getSatelliteTimeTs();
        Double longitude = firePointQo.getLongitude();
        Double latitude = firePointQo.getLatitude();
        String satelliteType = firePointQo.getSatelliteType();
        String landType = firePointQo.getLandType();
        String confidence = firePointQo.getConfidence();
        String fireImage = firePointQo.getFireImage();
        String satelliteImage = firePointQo.getSatelliteImage();
        String townCode = firePointQo.getTownCode();

        LocalDateTime satelliteLocalDateTime = LocalDateTime.ofEpochSecond(satelliteTimeTs, 0, ZoneOffset.ofHours(8));
        String satelliteTime = DateTimeUtils.localDateTimeToString(satelliteLocalDateTime);//卫星时间
        String addTime = DateTimeUtils.localDateTimeToString(LocalDateTime.now());//添加时间

        List<FirePointChannelConfigEntity> firePointChannelConfigEntityList = channelConfigDao.findAll();
        boolean inPreciseTimeRange = DispatchFirePointUtils.checkIfInPreciseTimeRange(satelliteLocalDateTime, firePointChannelConfigEntityList);
        //如果现在属于精准渠道的时间范围，那么精准渠道来的火点就入库精准火点库
        if (inPreciseTimeRange) {
            FirePointPreciseEntity firePointPreciseEntity = new FirePointPreciseEntity();
            firePointPreciseEntity.setFireCode(fireCode);
            firePointPreciseEntity.setCountyCode(String.valueOf(countyCode));
            firePointPreciseEntity.setCountyName(countyName);
            firePointPreciseEntity.setSatelliteTime(satelliteTime);
            firePointPreciseEntity.setLongitude(longitude);
            firePointPreciseEntity.setLatitude(latitude);
            firePointPreciseEntity.setSatelliteType(satelliteType);
            firePointPreciseEntity.setLandType(landType);
            firePointPreciseEntity.setConfidence(confidence);
            firePointPreciseEntity.setFireImage(fireImage);
            firePointPreciseEntity.setSatelliteImage(satelliteImage);
            firePointPreciseEntity.setTownCode(townCode);
            firePointPreciseEntity.setAddTime(addTime);
            bindFirePointAddress(firePointPreciseEntity);
            firePointPreciseDao.save(firePointPreciseEntity);
        }

        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "添加成功", locale);
    }

    private void bindFirePointAddress(FirePointPreciseEntity firePointPreciseEntity) {
        String countyCode = firePointPreciseEntity.getCountyCode();
        String countyName = firePointPreciseEntity.getCountyName();
        String townCode = firePointPreciseEntity.getTownCode();
        GeoCodeHelper geoCodeHelper = new GeoCodeHelper();
        geoCodeHelper.doGeoCode(firePointPreciseEntity.getLongitude(), firePointPreciseEntity.getLatitude());
        boolean geoCodeResult = geoCodeHelper.checkGeoCodeResult(countyCode, countyName, townCode);
        if (geoCodeResult) {
            firePointPreciseEntity.setTownCode(geoCodeHelper.getCalTownCode());
            firePointPreciseEntity.setTownName(geoCodeHelper.getCalTownName());
            firePointPreciseEntity.setFirePointAddress(geoCodeHelper.getCalFirePointAddress());
            firePointPreciseEntity.setRemark("");
        } else {
            firePointPreciseEntity.setFirePointAddress(geoCodeHelper.getCalFirePointAddress());
            String remark = "接收到的countyCode=" + countyCode + "，计算得到的countyCode=" + geoCodeHelper.getCalCountyCode() + "。接收到的countyName=" + countyName + "，计算得到的countyName=" + geoCodeHelper.getCalCountyName() + "。接收到的townCode=" + townCode + "，计算得到的townCode=" + geoCodeHelper.getCalTownCode() + "。";
            firePointPreciseEntity.setRemark(remark);
        }
    }

    private void bindFirePointAddress(FirePointOrdinaryEntity firePointOrdinaryEntity) {
        String countyCode = firePointOrdinaryEntity.getCountyCode();
        String countyName = firePointOrdinaryEntity.getCountyName();
        String townCode = firePointOrdinaryEntity.getTownCode();
        GeoCodeHelper geoCodeHelper = new GeoCodeHelper();
        geoCodeHelper.doGeoCode(firePointOrdinaryEntity.getLongitude(), firePointOrdinaryEntity.getLatitude());
        boolean geoCodeResult = geoCodeHelper.checkGeoCodeResult(countyCode, countyName, townCode);
        if (geoCodeResult) {
            firePointOrdinaryEntity.setTownCode(geoCodeHelper.getCalTownCode());
            firePointOrdinaryEntity.setTownName(geoCodeHelper.getCalTownName());
            firePointOrdinaryEntity.setFirePointAddress(geoCodeHelper.getCalFirePointAddress());
            firePointOrdinaryEntity.setRemark("");
        } else {
            firePointOrdinaryEntity.setFirePointAddress(geoCodeHelper.getCalFirePointAddress());
            String remark = "接收到的countyCode=" + countyCode + "，计算得到的countyCode=" + geoCodeHelper.getCalCountyCode() + "。接收到的countyName=" + countyName + "，计算得到的countyName=" + geoCodeHelper.getCalCountyName() + "。接收到的townCode=" + townCode + "，计算得到的townCode=" + geoCodeHelper.getCalTownCode() + "。";
            firePointOrdinaryEntity.setRemark(remark);
        }
    }

}
